import java.awt.*;
import java.util.*;
import java.io.*;
import java.net.*;
import java.rmi.*;

public class ChatContributorApplet extends java.applet.Applet
{
   static final int  UI_STATE_CONNECTED     = 1;
   static final int  UI_STATE_DISCONNECTED = 2;

   // UI Elements
   TextField      txtfldUserName;
   TextField      txtfldContribution;
   Label          lblUserName;
   Label          lblContribution;
   Button         btnConnect;
   Button         btnDisconnect;
   Button         btnSend;

   // variables used to signal the running monitor thread
   boolean           tryToConnect    = false;
   boolean           validConnection = false;

   String            dataToSend;
   boolean           dataReady       = false;

   ChatManager       cm;

   String             chatHostServer;

   // Applet method override
   public void init()
   {
      super.init();

      buildUI();

   }  // init


   public boolean handleEvent( Event e )
   {
      // Request to Send Data
      if ( ( (e.target == txtfldContribution) || (e.target == btnSend) ) && ( e.id == Event.ACTION_EVENT ) )
      {
         // Send message to chat server!
         if ( validConnection )
         {

            System.out.println( "Sending: " + txtfldUserName.getText() + ":" + txtfldContribution.getText() );

            try
            {
               cm.postMessage( txtfldUserName.getText() + ":" + txtfldContribution.getText() );
            }
            catch ( Exception e1 )
            {
               cm = null;
               System.out.println( "ChatContributor...Connection Failed" );
               validConnection = false;
               setUIState( UI_STATE_DISCONNECTED );
               System.out.println( "" + e1 );
            }
         }

         return true;
      }  // Send Data

      // Request to make a connection
      if ( ( e.target == btnConnect ) && ( e.id == Event.ACTION_EVENT ) )
      {
         System.out.println( "ChatListener...Going to connect" );
         // Link to ChatServer
         URL base = getDocumentBase();

         chatHostServer = "//" + base.getHost() + ":" + getParameter( "chatServerPort" ) + "/ChatService";

         System.out.println( "ChatListener...Connecting to: " + chatHostServer );

         try
         {
            cm = (ChatManager)Naming.lookup( chatHostServer );
            
            System.out.println( "ChatListener...Connection OK" );
            
            validConnection = true;
            
            setUIState( UI_STATE_CONNECTED );
         }
         catch ( Exception e1 )
         {
            System.out.println( "ChatListener...Connection Failed" );
            System.out.println( "" + e1 );
            validConnection = false;
            setUIState( UI_STATE_DISCONNECTED );
         }
            return true;

      }  // Connect

      // Request to disconnect
      if ( ( e.target == btnDisconnect ) && ( e.id == Event.ACTION_EVENT ) )
      {
         cm = null;
         validConnection = false;
         setUIState( UI_STATE_DISCONNECTED );

         return true;

      }  // Disconnect

      return super.handleEvent( e );
   }

   public void paint( Graphics g )
   {
      if ( validConnection )
      {
         g.setColor( Color.green );
         g.fillOval( 610, 3, 14, 14 );
      }
      else
      {
         g.setColor( Color.red );
         g.fillOval( 610, 3, 14, 14 );
      }
   }


   // UI Methods
   void buildUI()
   {
      setLayout( null );
      setForeground( Color.black );
      setBackground( Color.lightGray  );
      addNotify();

      txtfldUserName    = new TextField( "<UserName>" );
      txtfldContribution = new TextField( "<TextToSend>" );

      lblUserName       = new Label( "User Name:");
      lblContribution   = new Label( "Send to Chat:" );

      btnConnect        = new Button( "Connect" );
      btnDisconnect     = new Button( "Disconnect" );
      btnSend           = new Button( "Send" );

      lblUserName.reshape(         5,   5,  80, 20 );
      lblContribution.reshape(     5,  30,  80, 20 );

      txtfldUserName.reshape(    90,   5, 500, 20 );
      txtfldContribution.reshape( 90,  30, 500, 20 );

      btnConnect.reshape(         5,  55,  80, 20 );
      btnDisconnect.reshape(     90,  55,  80, 20 );
      btnSend.reshape(          175,  55,  80, 20 );

      setUIState( UI_STATE_DISCONNECTED );

      add( txtfldUserName );
      add( txtfldContribution );
      add( btnConnect );
      add( btnDisconnect );
      add( btnSend );
      add( lblUserName );
      add( lblContribution );
   }  // buildUI


   void setUIState( int state )
   {
      if ( UI_STATE_CONNECTED == state )
      {
         txtfldUserName.setEditable( false );
         txtfldUserName.setBackground( Color.lightGray );

         txtfldContribution.setEditable( true );
         txtfldContribution.setBackground( Color.white );

         try
         {
            txtfldUserName.setEnabled( false );
            txtfldContribution.setEnabled( true );
            btnConnect.setEnabled( false );
            btnDisconnect.setEnabled( true );
            btnSend.setEnabled( true );
         }
         catch ( NoSuchMethodError nsme )
         {
         }
      }

      if ( UI_STATE_DISCONNECTED == state )
      {
         txtfldUserName.setEditable( true );
         txtfldUserName.setBackground( Color.white );

         txtfldContribution.setEditable( false );
         txtfldContribution.setBackground( Color.lightGray );
         try
         {
            txtfldUserName.setEnabled( true );
            txtfldContribution.setEnabled( false );
            btnConnect.setEnabled( true );
            btnDisconnect.setEnabled( false );
            btnSend.setEnabled( false );
         }
         catch ( NoSuchMethodError nsme )
         {
         }
      }

      repaint();

   }  // setUIState
}
