import java.awt.*;
import java.util.*;
import java.io.*;
import java.net.*;

import java.rmi.*;
import java.rmi.server.*;

public class ChatListenerApplet extends java.applet.Applet implements ChatListener
{
   static final int UI_STATE_CONNECTED    = 1;
   static final int UI_STATE_DISCONNECTED = 2;

   // UI Elements
   TextArea       textArea1;
   Button         btnConnect;
   Button         btnDisconnect;
   Button         btnClear;

   // variables used to signal the running monitor thread
   boolean           validConnection = false;

   ChatManager       cm;

   String             chatHostServer;
   int                echoPort = 62346;

   // Applet method override
   public void init()
   {
      super.init();

      buildUI();

      // Register ourselves as a remote service
      // This allows the server to gain access to the ChatListener interface
      try
      {
         UnicastRemoteObject.exportObject( this );
      }
      catch ( RemoteException re )
      {
         System.out.println( "ChatListener...RMI System failure" );
         System.out.println( "" + re );
      }
   }


   // Method from Chat Listener Interface
   public void postMessage( String message )
   {
      textArea1.appendText( message + "\n" );
   }


   // Handles events from AWT user actions
   public boolean handleEvent( Event e )
   {

      // Request to Connect
      if ( ( e.target == btnConnect ) && ( e.id == Event.ACTION_EVENT ) )
      {
         System.out.println( "ChatListener...Going to connect" );
         // Link to ChatServer
         URL base = getDocumentBase();

         chatHostServer = "//" + base.getHost() + ":" + getParameter( "chatServerPort" ) + "/ChatService";

         System.out.println( "ChatListener...Connecting to: " + chatHostServer );

         try
         {
            cm = (ChatManager)Naming.lookup( chatHostServer );
            System.out.println( "ChatListener...Service Lookup OK" );

            cm.registerListener( this );

            System.out.println( "ChatListener...Connection OK" );

            validConnection = true;
            setUIState( UI_STATE_CONNECTED );
         }
         catch ( Exception e1 )
         {
            System.out.println( "ChatListener...Connection Failed" );
            System.out.println( "" + e1 );
            validConnection = false;
            setUIState( UI_STATE_DISCONNECTED );
         }
            return true;

      }  // Connect

      // Request to Disconnect
      if ( ( e.target == btnDisconnect ) && ( e.id == Event.ACTION_EVENT ) )
      {
         try
         {
            cm.unregisterListener( this );

            cm = null;
            validConnection = false;
            setUIState( UI_STATE_DISCONNECTED );
         }
         catch ( Exception e1 )
         {
            System.out.println( "ChatListener...RMI Error while disconnecting" );
            System.out.println( "" + e1 );
         }

         return true;

      }  // Disconnect


      // Request to Clear the display area
      if ( ( e.target == btnClear ) && ( e.id == Event.ACTION_EVENT ) )
      {
         textArea1.setText("");
         repaint();
         return true;
      }

      return super.handleEvent( e );
   }  // Clear


   public void paint( Graphics g )
   {
      if ( validConnection )
      {
         g.setColor( Color.green );
         g.fillOval( 610, 3, 14, 14 );
      }
      else
      {
         g.setColor( Color.red );
         g.fillOval( 610, 3, 14, 14 );
      }
   }

   void setUIState( int state )
   {
      if ( UI_STATE_CONNECTED == state )
      {
         textArea1.setBackground( Color.white );
         try
         {
            btnConnect.setEnabled( false );
            btnDisconnect.setEnabled( true );
         }
         catch ( NoSuchMethodError nsme )
         {
         }
      }

      if ( UI_STATE_DISCONNECTED == state )
      {
         textArea1.setBackground( Color.lightGray );
         try
         {
            btnConnect.setEnabled( true );
            btnDisconnect.setEnabled( false );
         }
         catch ( NoSuchMethodError nsme )
         {
         }
      }

      repaint();
   }


   void buildUI()
   {
      setLayout( null );
      setForeground( Color.black    );
      setBackground( Color.lightGray );
      addNotify();

      textArea1 = new TextArea( "" );
      textArea1.setEditable( false );
      textArea1.setBackground( Color.white );

      btnConnect   = new Button( "Connect" );
      btnDisconnect = new Button( "Disconnect" );
      btnClear     = new Button( "Clear" );

      textArea1.reshape(       5,   5, 590, 250 );
      btnConnect.reshape(       5, 260,  80,  20 );
      btnDisconnect.reshape(  90, 260,  80,  20 );
      btnClear.reshape(      175, 260,  80,  20 );

      try
      {
         btnConnect.setEnabled( true );
         btnDisconnect.setEnabled( false );
      }
      catch ( NoSuchMethodError nsme )
      {
      }

      add( textArea1 );
      add( btnConnect );
      add( btnDisconnect );
      add( btnClear );

   }  // buildUI

}

