import java.net.*;
import java.io.*;

/**
 *	RMIServer is a foundation class for all RMIServer in the	MageLang RMI Course.
 *	It provides bootstrap help with command line processing.
 */


public class RMIServer
{
   public  static final int    MAX_PORT_NUMBER    = 65535;
   public  static final int    MIN_PORT_NUMBER    = 0;

   protected  int    REGISTRY_PORT;
   protected  String VERSION_DATE;
   protected  String SERVER_NAME;
   protected  String SERVER_DESCRIPTION;
   protected  int    VERSION_MINOR;
   protected  int    VERSION_MAJOR;

   // Command Variables
   public     int    registryPort;
   public     String hostName;

   public RMIServer(
                     String[] args,
                     int      REGISTRY_PORT,
                     String   VERSION_DATE,
                     String   SERVER_NAME,
                     String   SERVER_DESCRIPTION,
                     int      VERSION_MAJOR,
                     int      VERSION_MINOR
                     )
   {
      this.REGISTRY_PORT      = REGISTRY_PORT;
      this.VERSION_DATE       =  VERSION_DATE;
      this.SERVER_NAME        = SERVER_NAME;
      this.SERVER_DESCRIPTION = SERVER_DESCRIPTION;
      this.VERSION_MAJOR      = VERSION_MAJOR;
      this.VERSION_MINOR      = VERSION_MINOR;

      if ( !processCommands( args ) )
      {
         helpCommands();
         System.exit( 1 );
      }
      System.out.println( SERVER_NAME + " started." );
      System.out.println( "   Host Name is:   " + hostName );
      System.out.println( "   Port Number is: " + registryPort );

   }


   protected void helpCommands()
   {
   	System.out.println("");
   	System.out.println("--- " + SERVER_DESCRIPTION + " version: " + VERSION_MAJOR + "." + VERSION_MINOR );
   	System.out.println("");
   	System.out.println("Usage: " + SERVER_NAME + " [-s ServerName] [-p RegistryPortNumber]");
   	System.out.println("");
   	System.out.println("Options:");
   	System.out.println("");
   	System.out.println("  -h  HostName            Host Name for RMI Service.");
   	System.out.println("                          (defaults to local host)");
   	System.out.println("");
   	System.out.println("  -p  RegistryPortNumber  Server port used to listen for Clients" );
   	System.out.println("                          (range: 0 <= InputPortNumber < 65535) "     );
   	System.out.println("                          (defaults to " + REGISTRY_PORT + ")"         );

   }	// helpCommand


   protected boolean processCommands( String[] args )
   {
   	int       NDX           = 0;
      int       numberOfArgs  = args.length;
      String    sValue;
      int       iValue;
      boolean   processedOK = true;

      System.out.println( "RMIServer.processCommands" );
      // search for "?" and override all other processing

      for ( NDX = 0; NDX < ( numberOfArgs ); NDX++ )
      {
         if ( args[NDX].equals( "?" ) || args[NDX].equals( "/?" ))
         {
            return false;
         }

      }


      // process "-h" switch

      sValue = "127.0.0.1";
      for ( NDX = 0; NDX < ( numberOfArgs - 1); NDX++ )
      {
         if ( args[NDX].equals( "-h" ) )
         {
            sValue = args[ NDX + 1 ];
            break;
         }

      }

      hostName = sValue;


      // process "-p" switch
      iValue = REGISTRY_PORT;
      for ( NDX = 0; NDX < ( numberOfArgs - 1); NDX++ )
      {
         if ( args[NDX].equals( "-p" ) )
         {
            try
            {
               iValue = Integer.parseInt(args[ NDX + 1 ]);
            }
            catch ( NumberFormatException nfe )
            {
               processedOK = false;
            }
            break;
         }

      }
      if ( ( iValue < MIN_PORT_NUMBER ) || ( iValue > MAX_PORT_NUMBER ) )
      {
         processedOK = false;
         registryPort = REGISTRY_PORT;
         System.out.println("");
         System.out.println( SERVER_NAME + "... Registry Port Number out of range" );
      }
      else
      {
         registryPort = iValue;
      }

      return processedOK;

   }	// processCommands

}  // class RMIServer







